from typing import Type

import requests


class Translator:
    BASE_API_URL = None

    def __init__(self, settings):
        self.settings = settings

    def request_translation(self, string_to_translate: str):
        response = self.request_for_translation(string_to_translate)
        if response.ok:
            translated_data = self.process_response(response)
            return translated_data
        else:
            raise requests.exceptions.ConnectionError('Something web wrong with translation requesting.')

    def request_for_translation(self, string_to_translate: str):
        raise NotImplementedError()

    def process_response(self, response: requests.Response):
        raise NotImplementedError()


def get_translator_by_name(name) -> Type[Translator]:
    if name == 'Yandex':
        return YandexTranslator
    elif name == 'Google':
        return GoogleTranslator


class YandexTranslator(Translator):
    BASE_API_URL = 'https://translate.yandex.net/api/v1.5/tr.json/translate'

    def request_for_translation(self, string_to_translate):
        params = {'key': self.settings.api_key,
                  'lang': '-'.join([self.settings.source_lang, self.settings.target_lang]),
                  }
        data = {'text': string_to_translate}
        return requests.post(self.BASE_API_URL, params=params,
                             data=data
                             )

    def process_response(self, response):
        return response.json()['text'][0]


class GoogleTranslator(Translator):
    BASE_API_URL = 'https://translation.googleapis.com/language/translate/v2'

    def request_for_translation(self, string_to_translate):
        headers = {'Authorization': 'Bearer "{}"'.format(self.settings.api_key)}
        data = {'q': string_to_translate,
                'source': self.settings.source_lang,
                'target': self.settings.target_lang,
                'format': 'text'}
        return requests.post(self.BASE_API_URL, headers=headers, data=data)

    def process_response(self, response):
        return response.json()['data']['translations'][0]['translatedText']



import re


class LineProcessor:
    code_mark: str = '```'

    def __init__(self, settings, line: str):
        self.settings = settings
        self._line: str = line
        self.pattern = self.get_regexp(self.settings.source_lang)

    def is_code_block_border(self):
        if self._line == self.code_mark:
            return True
        return self._line.startswith(self.code_mark) and not self._line.endswith(self.code_mark)

    def line_can_be_translated(self):
        return not self.__is_single_code_line() and self.__is_untranslated_paragraph()

    def __is_untranslated_paragraph(self):
        return re.match(self.pattern, self._line) is not None

    def __is_single_code_line(self):
        return self._line.startswith(self.code_mark) and self._line.endswith(self.code_mark) and len(self._line) > 3

    @staticmethod
    def get_regexp(source_lang):
        if source_lang == 'ru':
            return r'^[а-яА-Я]+.*'
        elif source_lang == 'en':
            return r'^[a-zA-Z]+.*'
        else:
            return r'^\d+.*'



import pathlib

from md_translate.exceptions import DirectoryNotFoundException


class FilesWorker:
    def __init__(self, settings):
        self.settings = settings
        self.__folder_to_process: pathlib.Path = pathlib.Path(self.settings.path)
        self.__validate_folder()
        self.md_files_list: list = self.__get_md_files_list()

    def __validate_folder(self):
        if not self.__folder_to_process.exists():
            raise DirectoryNotFoundException('No such folder: {}!'.format(self.__folder_to_process))
        if not self.__folder_to_process.is_dir():
            raise NotADirectoryError('With selected path provided a file, not a folder!')

    def __get_md_files_list(self) -> list:
        md_files_list = []
        for link in self.__folder_to_process.iterdir():
            if link.suffix == '.md':
                md_files_list.append(link.name)
        if len(md_files_list) == 0:
            raise FileNotFoundError('There are no MD files found with provided path!')
        return md_files_list

    def get_files_to_translate(self):
        files_to_translate = [self.__folder_to_process.joinpath(file_name) for file_name in self.md_files_list]
        return files_to_translate
